﻿//////////////////////////////////////////////
// GlobalTimerManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class GlobalTimer ;
	class RenderContext ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Commons
#include <nilkinsCommon/Patterns/SingletonClass.h>

// Standards
#include <map>
#include <mutex>

/// Class ------------------------------------
	
namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT GlobalTimerManager : public nkCommon::SingletonClass<GlobalTimerManager>
	{
		friend class nkCommon::SingletonClass<GlobalTimerManager> ;

		private :

			// Constructor
			GlobalTimerManager () ;
		
			// Attributs
			// Liste des timers... par contexte
			std::map<RenderContext*, GlobalTimer*> _timerPerContext ;

			// Timer actif
			GlobalTimer* _activeTimer ;

			// Mutex
			std::mutex _mutex ;
			
		public :

			// Destructor
			~GlobalTimerManager () ;

			// Gestion map
			GlobalTimer* createOrRetrieve (RenderContext* forContext) ;

			// Activation contexte
			void prepareForContext (RenderContext* context) ;

			// Getters
			GlobalTimer* getActiveTimer () const ;
	} ;
}