﻿//////////////////////////////////////////////
// Environment.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkScripts
{
	class EnvironmentStrategy ;
	class Function ;
	class Namespace ;
	class Script ;
	class ScriptObjectReference ;
	class UserType ;

	enum class INTERPRETER ;

	struct ExecutionResult ;
}

/// Includes ---------------------------------

// nkScripts
#include "../Dll/DllDefines.h"

#include "Functions/FunctionDefines.h"

// nkMemory
#include <NilkinsMemory/Containers/Buffer.h>
#include <NilkinsMemory/Containers/BufferView.h>
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <string>
#include <unordered_map>

/// Class ------------------------------------

namespace nkScripts
{
	class DLL_SCRIPTS_EXPORT Environment final
	{
		private :

			// Attributes
			// Name de l'environment
			std::string _name ;
			// List des fonctions actives
			std::unordered_map<std::string, Function*> _activeFunctions ;
			// Et des types actifs
			std::unordered_map<std::string, UserType*> _activeTypes ;
			// Namespaces actifs
			std::unordered_map<std::string, Namespace*> _activeNamespaces ;

			// L'environnement interne
			EnvironmentStrategy* _env ;

			// Interpreter Type actuel
			INTERPRETER _interpreterType ;

		public :

			// Constructeur, destructeur
			Environment () ;
			~Environment () ;

			// Getters
			const ExecutionResult& getLastExecutionResult () const ;
			nkMemory::StringView getName () const ;

			// Setters
			void setEnvironmentFor (INTERPRETER interpretType) ;
			void setName (const nkMemory::StringView& name) ;

			// Exécution
			bool execute (const Script& script) ;
			void requestGarbageCollection () ;

			// Manipulation mémoire environnement
			void setVar (const nkMemory::StringView& name, bool value) ;
			void setVar (const nkMemory::StringView& name, int value) ;
			void setVar (const nkMemory::StringView& name, float value) ;
			void setVar (const nkMemory::StringView& name, double value) ;
			void setVar (const nkMemory::StringView& name, const char* value) ;
			void setVar (const nkMemory::StringView& name, const nkMemory::StringView& value) ;
			void setVar (const nkMemory::StringView& name, const ScriptObjectReference& value) ;
			void setObject (const nkMemory::StringView& name, const nkMemory::StringView& userTypeName, void* value) ;

			bool isVarSet (const nkMemory::StringView& name) ;

			bool getVar (const nkMemory::StringView& name, bool defaultValue) ;
			int getVar (const nkMemory::StringView& name, int defaultValue) ;
			float getVar (const nkMemory::StringView& name, float defaultValue) ;
			double getVar (const nkMemory::StringView& name, double defaultValue) ;
			nkMemory::StringView getVar (const nkMemory::StringView& name, const char* defaultValue) ;
			nkMemory::StringView getVar (const nkMemory::StringView& name, const nkMemory::StringView& defaultValue) ;
			ScriptObjectReference getVar (const nkMemory::StringView& name) ;
			void* getObject (const nkMemory::StringView& name, const nkMemory::StringView& userTypeName = nullptr) ;
			ScriptObjectReference getScriptFunction (const nkMemory::StringView& name) ;

			// Ajout de fonctions
			Function* setFunc (const nkMemory::StringView& name) ;
			bool isFuncSet (const nkMemory::StringView& name) ;
			Function* getFunc (const nkMemory::StringView& name) ;

			// Ajout de types persos
			UserType* setUserType (const nkMemory::StringView& name) ;
			bool isUserTypeSet (const nkMemory::StringView& name) ;
			UserType* getUserType (const nkMemory::StringView& name) ;

			// Ajout de namespaces
			Namespace* setNamespace (const nkMemory::StringView& name) ;
			bool isNamespaceSet (const nkMemory::StringView& name) ;
			Namespace* getNamespace (const nkMemory::StringView& name) ;

			// Call C++ -> Script
			bool callScriptFunction (const ScriptObjectReference& reference, const DataStack& args, DataStack& expectedOutput) ;
			nkMemory::Buffer serializeScriptObject (const ScriptObjectReference& reference, bool forwardUserDataOwnership = false) ;
			ScriptObjectReference deserializeScriptObject (const nkMemory::BufferView<unsigned char>& object) ;
	} ;
}

